/******************************************************************************/
/*  TIMER.C: Time Functions for 10Hz Clock Tick using Timer0                  */
/******************************************************************************/
/*  Inspired by a sample application from Keil Elektronik                     */
/******************************************************************************/
/*  Sample for WinARM by M.Thomas <eversmith@heizung-thomas.de>               */
/*  Target Philips LPC2129 ARM7TDMI-S on Keil MCB2100 Eval.-Board             */
/******************************************************************************/

#include "lpc21xx.h"
#include "config.h"
#include "keyboard.h"
#include "timer.h"

static volatile unsigned long timeval;
static volatile unsigned long delayval;
#define VICVectCntl0_ENABLE (1<<5)

#define VIC_Channel_Timer0  4

#define TxTCR_COUNTER_ENABLE (1<<0)
#define TxTCR_COUNTER_RESET  (1<<1)
#define TxMCR_INT_ON_MR0     (1<<0)
#define TxMCR_RESET_ON_MR0   (1<<1)
#define TxIR_MR0_FLAG        (1<<0)


void __attribute__ ((interrupt("IRQ"))) tc0_cmp(void);

/* Timer0 Compare-Match Interrupt Handler (ISR) */
void tc0_cmp(void) 
{
	timeval++;
	delayval++;
	//getKey();
	T0IR = TxIR_MR0_FLAG; // Clear interrupt flag by writing 1 to Bit 0
	VICVectAddr = 0;       // Acknowledge Interrupt (rough?)
}

/* Setup Timer0 Compare-Match Interrupt                      */
/* no prescaler timer runs at PCLK = FOSC*PLL_M / VPBDIV_VAL */
void init_timer (void) {
	
	T0MR0 = (PCLK/1000*10)-1;     // Compare-hit at 10mSec (-1 reset "tick")
	T0MCR = TxMCR_INT_ON_MR0 | TxMCR_RESET_ON_MR0; 	// Interrupt and Reset on MR0
	T0TCR = TxTCR_COUNTER_ENABLE;            // Timer0 Enable

	VICVectAddr0 = (unsigned long)tc0_cmp;   // set interrupt vector in 0
	VICVectCntl0 = VICVectCntl0_ENABLE | VIC_Channel_Timer0; // use it for Timer 0 Interrupt:
	VICIntEnable = (1<<VIC_Channel_Timer0);    // Enable Timer0 Interrupt
}

void delayUs(uint32_t delay_US)
{
	delayval=0;
	uint32_t startTime = T0TC+delayval*T0MR0;
	uint32_t volatile endTime=startTime+58*delay_US;
	if(delay_US>5) while (endTime > T0TC+delayval*T0MR0);
}

void delayMs(uint32_t delay_MS)
{
	delayval=0;
	uint32_t startTime = T0TC+delayval*T0MR0;
	uint32_t volatile endTime=startTime+58981*delay_MS;
	if(delay_MS>0) while (endTime > T0TC+delayval*T0MR0);
}

uint32_t getTimer(void)
{
	return timeval*T0MR0+T0TC;
}

void setTimer(uint32_t val)
{
	T0TC=0;
	timeval=val;
	T0TC=0;
}

