#define VERSION "V0.1"

#include "types.h"
//#include "LPC21xx.h"
#include "lpc2103.h"
#include "config.h"
#include "armVIC.h"
#include "timer.h"
#include "uart.h"
#include "printf_P.h"
#include "stdlib.h"
//#include "math.h"

void cursorPos(unsigned char y, unsigned char x);
void rahmen(unsigned char y, unsigned char x, unsigned char h, unsigned char b, unsigned char balken);
void hBalken(unsigned char y, unsigned char x, unsigned char b, unsigned char balken);
void farbe(unsigned char zeichen, unsigned char hgrund);


/******************************************************************************
 *
 * Function Name: lowInit()
 *
 * Description:
 *    This function starts up the PLL then sets up the GPIO pins before
 *    waiting for the PLL to lock.  It finally engages the PLL and
 *    returns
 *
 * Calling Sequence: 
 *    void
 *
 * Returns:
 *    void
 *
 *****************************************************************************/
static void lowInit(void)
{
	// set PLL multiplier & divisor.
	// values computed from config.h
	PLLCFG = PLLCFG_MSEL | PLLCFG_PSEL;
	
	// enable PLL
	PLLCON = PLLCON_PLLE;
	PLLFEED = 0xAA;                       // Make it happen.  These two updates
	PLLFEED = 0x55;                       // MUST occur in sequence.
	
	// setup the parallel port pin
//	IO0CLR = PIO0_ZERO_BITS;                // clear the ZEROs output
//	IO0SET = PIO0_ONE_BITS;                 // set the ONEs output
//	IO0DIR = PIO0_OUTPUT_BITS;              // set the output bit direction
	//IO0DIR = LED;
	SCS = 1;	// Fast-IO aktivieren
	FIO0DIR1 = BIT(6);
	//FIODIR = LED;

	
	// wait for PLL lock
	while (!(PLLSTAT & PLLSTAT_LOCK))
		continue;
	
	// enable & connect PLL
	PLLCON = PLLCON_PLLE | PLLCON_PLLC;
	PLLFEED = 0xAA;                       // Make it happen.  These two updates
	PLLFEED = 0x55;                       // MUST occur in sequence.
	
	// setup & enable the MAM
	MAMTIM = MAMTIM_CYCLES;
	MAMCR = MAMCR_FULL;
	
	// set the peripheral bus speed
	// value computed from config.h
	VPBDIV = VPBDIV_VALUE;                // set the peripheral bus clock speed
}

/******************************************************************************
 *
 * Function Name: sysInit()
 *
 * Description:
 *    This function is responsible for initializing the program
 *    specific hardware
 *
 * Calling Sequence: 
 *    void
 *
 * Returns:
 *    void
 *
 *****************************************************************************/
static void sysInit(void)
{
	lowInit();                            // setup clocks and processor port pins
	
	// set the interrupt controller defaults
#if defined(RAM_RUN)
	MEMMAP = MEMMAP_SRAM;                 // map interrupt vectors space into SRAM
#elif defined(ROM_RUN)
	MEMMAP = MEMMAP_FLASH;                // map interrupt vectors space into FLASH
#else
#error RUN_MODE not defined!
#endif

	VICIntEnClr = 0xFFFFFFFF;           // clear all interrupts
	VICIntSelect = 0x00000000;            // clear all FIQ selections
	VICDefVectAddr = (uint32_t)reset;     // point unvectored IRQs to reset()
	
	//  wdtInit();                            // initialize the watchdog timer
	//initSysTime();                        // initialize the system timer
	
	uart0Init(UART_BAUD(HOST_BAUD), UART_8N1, UART_FIFO_8); // setup the UART
	//uart1Init(UART_BAUD(HOST_BAUD), UART_8N1, UART_FIFO_8); // setup the UART
}

/******************************************************************************
 *
 * Function Name: main()
 *
 * Description:
 *    This function is the program entry point.  After initializing the
 *    system, it sends a greeting out UART0 then enters an endless loop
 *    echoing chracters on the UART and blinking an LED every half
 *    second.
 *
 * Calling Sequence: 
 *    void
 *
 * Returns:
 *    void
 *
 *****************************************************************************/
int main(void)
{
	//IO0CLR = LED;
	FIO0CLR1 = 1<<6;
	//FIOCLR = LED;
	timercounter=0;
	sysInit();
	initTimer();

	enableIRQ();	

	setChannel(0);
	farbe(0,0);
	
	uart0Puts("\033[2J"); // Bildschirm lschen
	uart0Puts("\033[?25l");// Kursor aus

	farbe(7,0);
	rahmen(1, 1, 40, 80, 1);
	hBalken(3, 1, 80, 1);
	farbe(6,0);
	rahmen(7, 4, 3, 74, 0);
	farbe(4,0);
	rahmen(6, 3, 5, 76, 0);
	
	cursorPos(2,5);
	farbe(3,0);
	printf("IO, UART- and Timer-Interrupt Tests for MP2103-Stick " VERSION);
	//delayMs(10);
	cursorPos(4,5);
	farbe(2,0);
	printf("Quarz: %d  MAMTIM: %d  CCLK: %dMHz  PCLK: %dMHz  PBSD: %d", FOSC/1000, MAMTIM, CCLK/1000000, PCLK/1000000, PBSD);

	farbe(1,0);
	cursorPos(8,10);
	printf("Taste druecken");
	
	srand(12487);
	//IO0CLR = BIT(14);
	//IO0SET = BIT(14);
	for (;;)	// ever
	{	
		int ch;
		if ((ch = uart0Getch()) >= 0)
		{
			farbe(1,0);
			cursorPos(8,10);
			printf("Taste ");
			if(ch>=32) printf("%c ", ch);
			printf("(%d)  \033[s                                   \033[u", ch, ch);
		}
		if(ch==27)
			while ((ch = uart0Getch()) >= 0)
			{
				if(ch==27) cursorPos(8,10);
				printf("Taste %c (%d)  ", ch, ch);
			}

		delayMs(100);
		//IO0PIN = IO0PIN ^ LED;
		FIO0PIN1 = FIO0PIN1 ^ (1<<6);
		//FIO0CLR1 = 6;
		//FIOPIN = FIOPIN ^ LED;
		
		farbe(5,0);
		cursorPos(12,4);
		printf("Port: %08X", FIOPIN);
		
		farbe(7,0);
		cursorPos(14,4);
		printf("timercounter: %10ld", timercounter);
		
	} // for
	
	return 0;
}

void cursorPos(unsigned char y, unsigned char x)
{
	if( (y<41) & (x<81) )
	printf("\033[%d;%dH", y, x);// Cursor-Pos
}

void rahmen(unsigned char y, unsigned char x, unsigned char h, unsigned char b, unsigned char balken)
{
	unsigned char ypos, xpos;
	balken&=1;//begrenzen
	if( ((y+h-1)<41) & ((x+b-1)<81) )
	{
		//horizontaler Balken oben und unten
		for(xpos=x+1; xpos<(x+b-1); xpos++)
		{
			printf("\033[%d;%dH%c", y, xpos, 196+balken*9);// Cursor-Pos + Zeichen
			printf("\033[%d;%dH%c", y+h-1, xpos, 196+balken*9);// Cursor-Pos + Zeichen
		}
		for(ypos=y+1; ypos<(y+h-1); ypos++)
		{
			printf("\033[%d;%dH%c", ypos, x, 179+balken*7);// Cursor-Pos + Zeichen
			printf("\033[%d;%dH%c", ypos, x+b-1, 179+balken*7);// Cursor-Pos + Zeichen
		}
		printf("\033[%d;%dH%c", y, x, 218-balken*17);// Cursor-Pos + Zeichen
		printf("\033[%d;%dH%c", y, x+b-1, 191-balken*4);// Cursor-Pos + Zeichen
		printf("\033[%d;%dH%c", y+h-1, x, 192+balken*8);// Cursor-Pos + Zeichen
		printf("\033[%d;%dH%c", y+h-1, x+b-1, 217-balken*29);// Cursor-Pos + Zeichen

	}
}

void hBalken(unsigned char y, unsigned char x, unsigned char b, unsigned char balken)
{
	unsigned char xpos;
	balken&=1;//Wert begrenzen
	if( (y<41) & ((x+b-1)<81) )
	{
		//horizontaler Balken oben und unten
		for(xpos=x+1; xpos<(x+b-1); xpos++)
		{
			printf("\033[%d;%dH%c", y, xpos, 196+balken*9);// Cursor-Pos + Zeichen
		}
		printf("\033[%d;%dH%c", y, 1, 195+balken*9);// Cursor-Pos + Zeichen
		printf("\033[%d;%dH%c", y, x+b-1, 180+balken*5);// Cursor-Pos + Zeichen
	}
}

void farbe(unsigned char zeichen, unsigned char hgrund)
{
	hgrund&=7;//Wert begrenzen
	zeichen&=7;//Wert begrenzen
	printf("\033[1;7;%d;%dm", hgrund+40, zeichen+30);//farbe
}
